/********************************************************************
 * tile map management definitions
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 *******************************************************************/

#ifndef	_BACKGROUND_H
#define	_BACKGROUND_H

#include "types.h"
#include "gba_bkg.h"
#include "math.h"
#include "object.h"


// settings bits for an instance of a background tile object
#define	BKGOBJSET_FLIPX				TILE_FLIP_X	// flip on x axis (must be same as hardware tile ref bit)
#define	BKGOBJSET_FLIPY				TILE_FLIP_Y	// flip on y axis (must be same as hardware tile ref bit)
#define	BKGOBJSET_FLIPBITS			(BKGOBJSET_FLIPX|BKGOBJSET_FLIPY)
#define	BKGOBJSET_ANIMOBJ			0x8000 		// tells the system this object has an associated dynamic object
#define	BKGOBJSET_ANIMOBJNUM_MASK	0x00FF 		// bottom bits are the dynamic obj number, if anim flag is set, up to 256 in world


// background object struct
// - this struct is used to store info about a background (where its tiles are located,
//   where the tile map is located, its scroll settings, etc)
typedef struct BackgroundStruct
{
	s24f8_vect2D	pos;	// world position of this background (top-left corner of screen)

	s24f8_vect2D	delta;		// how far to shift by this frame

	u16				scrollX, scrollY;	// hardware scroll position for this background

	u32				*pTiles;	// address where the tiles for this background are located
	u16				*pMap;		// address where the tile map for this background are located

	u8				bkgNum;		// which hardware background this struct is associated with (0..3)

} BackgroundStruct;


// graphical frame data structure
// - stores details about a single background graphical picture
typedef struct BackgroundFrameStruct
{
	// NOTE: first pass, assume all background tiles are in VRAM... no need for tile management

	u8 sizeX, sizeY;	// tile size of the frame (first pass, 0 = illegal... 0 = 256?)

	const u16 *pMap;	// pointer to the tile map data for the graphical frame

} BackgroundFrameStruct;



extern BackgroundStruct gBkg0;	// background 0 data


/***********************************************************
 * init the global background settings
 **********************************************************/
extern void Background_InitSystem (void);


/*************************************************************************************************
 * initialize a background
 *
 * IN:		bkgNum - which background is being initialized (0..3)
 * 			tileBlock - character block number (0..3) - what block in VRAM the tiles for the background go
 * 			mapBlock - screen block number (0..31) - what block in VRAM the tile map will go
 * 			priority - priority setting (0..3)
 *          flags - settings flags (BKG_MOSAIC, BKG_COLOR_MODE, BKG_WRAP)
 * 			size - tile map size (0..3) - see below
 *
 * RETURNS:	ptr to the newly created background object
 *
 * NOTES:	1) this routine does NOT turn on the display of the tile map in the display mode register
 * 			2) possible size settings:
 *				0 = 32x32 tiles, or 16x16 if rotating background
 *				1 = 64x32 tiles, or 32x32 if rotating background
 *				2 = 32x64 tiles, or 64x64 if rotating background
 *				3 = 64x64 tiles, or 128x128 if rotating background
 *			3) flags:
 * 				BKG_MOSAIC - if set, the background uses mosaic mode
 * 				BKG_COLOR_MODE - if set, the background uses a 256 color palette (default is 16 color palettes)
 * 				BKG_WRAP - if set, the background wraps at the edges (for rotating backgrounds)
 *************************************************************************************************/
extern BackgroundStruct *Background_New (u8 bkgNum, u8 tileBlock, u8 mapBlock, u8 priority, u16 flags, u8 size);


#endif

