/**************************************************************
 * core object system headers
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 *************************************************************/

#ifndef _OBJECT_H
#define	_OBJECT_H

#include "types.h"


#define OBJECT_HEADER	u16 type; u16 nID; u16 prev; u16 next;

#define	OBJECT_NULL		0xFFFF

#define	MAX_OBJECTS			64	// total number of objects in the world
#define	OBJECT_BYTE_SIZE	64	// byte size of each game object
// total size of object buffer = the multiple of these two (32K)

// each object is defined by a unique 32 bit id/handle.  The top word is a unique object number, the
// bottom word is the objects index into the object buffer.  A value of 0 is invalid, and is used
// to show that an object handle has become invalid
#define	OBJECT_ID_NULL		0
#define	OBJECT_UNUSED_NID	0


// object types used by this game
enum eObjectTypes
{
	OBJECT_DUDE,	// dude character object
	OBJECT_BULLET,	// bullet object
	OBJECT_POWERUP,	// powerup object

	NUM_OBJECT_TYPES,
} eObjectTypes;

// basic dynamic game object data structure
typedef struct ObjectStruct
{
	u16		type;		// object type
	u16     nID;		// unique id for this object
	u16		prev, next;	// links to the previous and next objects in list

	u8		unused[OBJECT_BYTE_SIZE-8];	// object type specific data (total size of struct is OBJECT_BYTE_SIZE)
} ObjectStruct;

// definition of the generic object update function
typedef u32 (*ObjectUpdateFunction)	(ObjectStruct *pObj);


// buffer of game objects
extern ObjectStruct *gpObjectBuffer;

// buffer of active objects (sorted by object type)
u16 activeObjects[NUM_OBJECT_TYPES];


/************************************************************************
 * get the pointer to an object from its handle
 *
 * IN:		32 bit handle to the object
 *
 * NOTE:	no error or dead object checking!
 ************************************************************************/
#define	Object_getPtrS(id)		(&gpObjectBuffer[id & 0x0000FFFF])


/************************************************************************
 * check if there is ay active objects of the input type
 *
 * IN:		type of object
 *
 * RETURNS:	non-zero if any active objects
 ************************************************************************/
#define	Object_CheckTypeActive(type)		(activeObjects[type] != OBJECT_NULL)


/************************************************************************
 * get the pointer to an object from its handle
 *
 * IN:		32 bit handle to the object
 *
 * RETURNS:	ptr to the handle's ObjectStruct, NULL if object has died
 ************************************************************************/
extern ObjectStruct *Object_getPtr (u32 id);


/************************************************************************
 * initialize the object system
 ***********************************************************************/
extern void Object_InitSystem (void);


/************************************************************************
 * get a new game object
 *
 * IN:		type of object
 *
 * RETURNS:	handle to the object
 *
 * NOTE:	the handle for an object is a 32 bit value:
 * 				lower 16 bits = object number in object buffer
 * 				upper 16 bits = object's nID (useful for tracking if the object has died)
 ************************************************************************/
extern u32 Object_New (u16 type);


/******************************************************************************
 * deletes an object from the system (returns it to the unused buffer)
 *
 * IN:		object handle
 *****************************************************************************/
extern void Object_Kill (u32 id);


/******************************************************************************
 * update all the active objects
 *****************************************************************************/
extern void Object_UpdateAll (void);

#endif

